<?xml version="1.0"?>
<!-- Copyright (c) 2004 Victor Porton. All rights reserved.
     It is not a freeware. See the license.
     Send any changes/patches to Victor Porton <porton@ex-code.com>.
     See http://ex-code.com/rss-merge/ for additional info.
     If you desire to include this file as a part of your
     software package or software distribution, please contact me. -->
<!DOCTYPE xsl:stylesheet>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
                xmlns    ="http://purl.org/rss/1.0/"
                xmlns:rss="http://purl.org/rss/1.0/"
                xmlns:dc ="http://purl.org/dc/elements/1.1/"
		xmlns:xi="http://www.w3.org/2001/XInclude"
                xmlns:sub    ="http://purl.org/rss/1.0/modules/subscription/"
                xmlns:taxo   ="http://purl.org/rss/1.0/modules/taxonomy/"
                xmlns:dcterms="http://purl.org/dc/terms/"
                xmlns:admin="http://webns.net/mvcb/"
                xmlns:merge="http://ex-code.com/design/merge-rss/"
                version="1.1"
                exclude-result-prefixes="xi merge rss sub">

  <!--
    mod_subscription is extended with @merge:stop attribute:
    <sub:channel merge:stop="true"> is pasted literally instead
    of to be processed.
  -->

  <!-- Specify param-rss-max-items=true() for unlimited. -->
  <xsl:param name="param-rss-max-items" select="15"/>

  <xsl:output method="xml"
              version="1.0"
              indent="yes"
              encoding="utf-8"
              media-type="application/rss+xml"/>

  <!--xsl:key name="about" match="*[@rdf:about]" use="@rdf:about"/>
  <xsl:key name="ID"    match="*[@rdf:ID]"    use="concat('#',@rdf:ID)"/-->

  <xsl:template match="/rdf:RDF">
    <xsl:if test="system-property('xsl:version')&lt;1.1">
      <xsl:message terminate="yes">
        <xsl:text>This stylesheet requires XSLT 1.1 or above.</xsl:text>
      </xsl:message>
    </xsl:if>
    <xsl:variable name="flat">
      <xsl:apply-templates mode="include" select="*|rss:channel/sub:channel"/>
    </xsl:variable>
    <xsl:variable name="items"
                  select="$flat/rss:item|$flat/rss:image|$flat/taxo:topic"/>
    <!--xsl:variable name="items" select="$items1[@rdf:about|@rdf:ID]"/-->
    <xsl:variable name="unique-items">
      <xsl:for-each select="$items">
        <xsl:variable name="about">
          <xsl:value-of select="@rdf:about"/>
          <xsl:if test="@rdf:ID">
            <xsl:value-of select="concat('#',@rdf:ID)"/>
          </xsl:if>
        </xsl:variable>
        <xsl:if test="$about!=''">
          <!-- Commented because of some tricky bug. -->
          <!--xsl:if test="generate-id()=generate-id(key('about',$about)|key('ID',$about))"-->
          <xsl:if test="not(preceding-sibling::*[@rdf:about=$about or concat('#',@rdf:ID)=$about])">
            <xsl:copy-of select="."/>
          </xsl:if>
        </xsl:if>
      </xsl:for-each>
    </xsl:variable>
    <xsl:variable name="sorted-items">
      <xsl:for-each select="$unique-items/*">
        <!-- This ensures <channel> before <item> before <topic> -->
        <xsl:sort select="local-name()"
                  lang="en"
                  data-type="text"
                  case-order="lower-first"/>
        <xsl:sort select="dc:date|dcterms:issued"
                  lang="en"
                  data-type="text"
                  order="descending"
                  case-order="upper-first"/>
        <xsl:copy-of select="."/>
      </xsl:for-each>
    </xsl:variable>
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:variable name="real-items"
                    select="$sorted-items[position()&lt;$param-rss-max-items]"/>
      <channel>
        <xsl:for-each select="rss:channel">
          <xsl:apply-templates select="@*|*"/>
        </xsl:for-each>
        <admin:generatorAgent rdf:resource="http://ex-code.com/rss-merge/"/>
        <items>
          <rdf:Seq>
            <xsl:for-each select="$real-items/rss:item">
              <xsl:variable name="about">
                <xsl:value-of select="@rdf:about"/>
                <xsl:if test="@rdf:ID">
                  <xsl:value-of select="concat('#',@rdf:ID)"/>
                </xsl:if>
              </xsl:variable>
              <rdf:li rdf:resource="{$about}"/>
            </xsl:for-each>
          </rdf:Seq>
        </items>
      </channel>
      <xsl:apply-templates select="$flat/self::*[not(rss:item|rss:image|taxo:topic)]"/>
      <!--xsl:variable name="items-to-process">
        <mode:mode mode:mode="priv:items-mode">
          <mode:argument>
            <xsl:copy-of select="$sorted-items/rss:item"/>
          </mode:argument>
          <xsl:copy-of select="$sorted-items[position()&lt;$param-rss-max-items]"/>
        </mode:mode>
      </xsl:variable-->
      <xsl:copy-of select="$real-items"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="rss:items"/>

  <xsl:template match="sub:channel"/>

  <xsl:template match="admin:generatorAgent"/>

  <!--xsl:template match="mode:mode[mode:mode='priv:items-mode' and rss:items]">
    <xsl:for-each select="rss:items">
      <items>
        <xsl:copy-of select="@*"/>
        <rdf:Seq>
          <xsl:copy-of select="rdf:Seq/@*"/>
          <xsl:for-each select="rdf:Seq/rdf:li">
            <xsl:if test="key('about',@resource)|key('ID',@resource)">
              <xsl:copy-of select="."/>
            </xsl:if>
          </xsl:for-each>
        </rdf:Seq>
      </items>
    </xsl:template>
  </xsl:template-->

  <!--xsl:template match="mode:mode[mode:mode='priv:items-mode' and count(*)=1]"
                priority="-10">
    <xsl:apply-templates select="*"/>
  </xsl:template-->

  <!-- mod_subscription support -->
  <xsl:template mode="include" match="sub:channel[@merge:stop='true']">
    <xsl:copy-of select="@*[name()!='http://ex-code.com/design/merge-rss/:stop']|*"/>
  </xsl:template>

  <xsl:template mode="include" match="sub:channel">
    <!--xsl:if test="dc:format='' or dc:format='application/rss+xml'"-->
      <xsl:for-each select="document(sub:site/@rdf:resource,.)/rdf:RDF">
        <xsl:apply-templates mode="include"
                             select="rss:item|rss:image|taxo:topic|rss:channel/sub:channel"/>
      </xsl:for-each>
    <!--/xsl:if-->
  </xsl:template>

  <!-- Partial XInclude support (only <xinclude href="..."/>) -->
  <xsl:template mode="include" match="xi:xinclude">
    <xsl:if test="@xpointer">
      <xsl:message terminate="yes">
        <xsl:text>Merge RSS doesn't support xpointer</xsl:text>
      </xsl:message>
    </xsl:if>
    <xsl:apply-templates mode="include" select="document(@href,.)"/>
  </xsl:template>

  <xsl:template match="*|text()|@*">
    <xsl:copy>
      <xsl:apply-templates select="*|text()|@*"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template mode="include" match="*|text()|@*">
    <xsl:copy>
      <xsl:apply-templates mode="include" select="*|text()|@*"/>
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>

